@@ -18,15 +18,23 @@ import functools
 import itertools
 from os import PathLike
 from typing import (Any, Callable, Dict, FrozenSet, Iterable, Iterator, List,
-                    Optional, Sequence, Tuple, Union)
+                    Mapping, Optional, Sequence, Tuple,
+                    TypeVar, Union)
 from uuid import UUID
+from xxlimited import Str
 
 import numpy as np
 import pandas as pd
 
+from instancelib.environment.memory import MemoryEnvironment
+from instancelib.labels.memory import MemoryLabelProvider
+
 from ..environment import AbstractEnvironment
 from ..environment.text import TextEnvironment
-from ..instances.text import MemoryTextInstance
+from ..instances.base import Instance, InstanceProvider
+from ..instances.text import MemoryTextInstance, TextInstanceProvider
+from ..labels.base import LabelProvider
+from ..typehints import DT, KT, LT, RT, VT
 from ..utils.func import list_unzip3, single_or_collection
 
 
@@ -300,6 +308,45 @@ def pandas_to_env_with_id(df: Union[pd.DataFrame, Dict[str, pd.DataFrame]],
         env = build_environment_with_id(df, identity_mapper, labels, id_col,l_data_cols, l_label_cols)
     return env
 
+IT = TypeVar("IT", bound="Instance[Any, Any, Any, Any]")
+
+def instance_extractor(df: pd.DataFrame, 
+                       id_extractor: Callable[[pd.Series, Any], KT],
+                       data_extractor: Callable[[pd.Series], DT],
+                       vector_extractor: Callable[[pd.Series], VT],
+                       repr_extractor: Callable[[pd.Series], RT],
+                       label_extractor: Callable[[pd.Series], FrozenSet[LT]],
+                       builder: Callable[[KT, DT, VT, RT, pd.Series, Any], IT]
+                       ) -> Iterator[Tuple[KT, IT, FrozenSet[LT]]]: 
+    def row_func(row: pd.Series, idx: Any) -> Tuple[KT, IT, FrozenSet[LT]]:
+        identifier = id_extractor(row, idx)
+        data = data_extractor(row)
+        vector = vector_extractor(row)
+        representation = repr_extractor(row)
+        labels = label_extractor(row)
+        instance = builder(identifier, data, vector, representation, row, idx)
+        return identifier, instance, labels
+    for idx, row in df.iterrows():
+        yield row_func(row, idx)
+
+
+                    
+def to_dicts(triples: Iterator[Tuple[KT, IT, FrozenSet[LT]]]) -> Tuple[Mapping[KT, IT], Mapping[KT, FrozenSet[LT]]]:
+    keys, instances, labels = list_unzip3(triples)
+    instance_dict = dict(zip(keys, instances))
+    label_dict = dict(zip(keys, labels))
+    return instance_dict, label_dict
+
+def to_environment(
+        prov_builder: Callable[[Mapping[KT, IT]], InstanceProvider[IT, KT, DT, VT, RT]],
+        labelprov_builder: Callable[[Mapping[KT, FrozenSet[LT]]], LabelProvider[KT, LT]],
+        dictionaries: Tuple[Mapping[KT, IT], Mapping[KT, FrozenSet[LT]]]
+        )-> AbstractEnvironment[IT, KT, DT, VT, RT, LT]:
+    instances, labels = dictionaries
+    ins_provider = prov_builder(instances)
+    lbl_provider = labelprov_builder(labels)
+    return MemoryEnvironment(ins_provider, lbl_provider)
+
 
 def build_from_multiple_dfs(df_dict: Dict[str, pd.DataFrame], 
                             label_mapper: Callable[[Any], Optional[str]],
@@ -395,4 +442,70 @@ def build_from_multiple_dfs_with_ids(df_dict: Dict[str, pd.DataFrame],
         [])
     for key, split_indices in indices_table.items():
         environment[key] = environment.create_bucket(split_indices)
-    return environment
\ No newline at end of file
+    return environment
+
+
+def one_hot_encoded_extractor(*cols: str) -> Callable[[pd.Series], FrozenSet[str]]:
+    def callable(row: pd.Series) -> FrozenSet[str]:
+        lbls = frozenset([col for col in cols if row[col]])
+        return lbls
+    return callable
+
+def text_concatenation(*cols: str) -> Callable[[pd.Series], str]:
+    def callable(row: pd.Series) -> str:
+        text = " ".join([col for col in cols if row[col]])
+        return text
+    return callable
+
+def no_vector() -> Callable[[pd.Series], Optional[np.ndarray]]:
+    def callable(row: pd.Series) -> Optional[np.ndarray]:
+        return None
+    return callable
+
+def id_col(col: str) -> Callable[[pd.Series, Any], Any]:
+    def callable(row: pd.Series, idx: Any) -> Any:
+        identifier = row[col]
+        return identifier
+    return callable
+
+def id_index() -> Callable[[pd.Series, Any], Any]:
+    def callable(row: pd.Series, idx: Any) -> Any:
+        return idx
+    return callable
+
+def id_index_prefix(prefix: str) -> Callable[[pd.Series, Any], str]:
+    def callable(row: pd.Series, idx: Any) -> str:
+        return f"{prefix}_{idx}"
+    return callable
+
+def text_builder(identifier: KT,
+                 data: str, 
+                 vector: VT, 
+                 representation: str, 
+                 row: pd.Series,
+                 idx: Any) -> MemoryTextInstance[KT, VT]:
+    return MemoryTextInstance(identifier, data, vector, representation)
+
+
+def text_from_pandas_multilabel(
+    df_dict: Dict[str, pd.DataFrame],
+    text_cols: Sequence[str],
+    label_cols: Sequence[str],
+    labelset: FrozenSet[str]):
+    def instances() -> Iterator[Tuple[str, MemoryTextInstance[str, Any], FrozenSet[str]]]:
+        for name, df in df_dict.items():
+            df_instances = instance_extractor(
+                df, 
+                id_index_prefix(name), 
+                text_concatenation(*text_cols),
+                no_vector(),
+                text_concatenation(*text_cols),
+                one_hot_encoded_extractor(*label_cols),
+                text_builder)
+            yield from df_instances
+    dicts = to_dicts(instances())
+    pass
+
+
+
+
